@@ -15,10 +15,7 @@ import signal
 import time
 import io
 import subprocess as sp
-import warnings
 import tempfile
-from contextlib import closing
-from operator import attrgetter
 
 from . import api as pg_api
 from . import configfile
@@ -437,7 +434,9 @@ class Cluster(pg_api.Cluster):
 				"cannot connect if cluster is not running",
 				creator = self
 			)
-		return self.connection(**kw).connect()
+		x = self.connection(**kw)
+		x.connect()
+		return x
 
 	def address(self):
 		"""
