@@ -5,11 +5,11 @@
 import sys
 import os
 import unittest
+import gc
 import threading
 import time
 import datetime
 import decimal
-import gc
 from itertools import chain, islice
 from operator import itemgetter
 
@@ -17,8 +17,9 @@ from ..python.datetime import FixedOffset
 from .. import types as pg_types
 from .. import exceptions as pg_exc
 from .. import unittest as pg_unittest
+from .. import lib as pg_lib
 
-type_samples = (
+type_samples = [
 	('smallint', (
 			((1 << 16) // 2) - 1, - ((1 << 16) // 2),
 			-1, 0, 1,
@@ -267,7 +268,39 @@ type_samples = (
 			pg_types.varbit('010111101111'),
 		],
 	),
-)
+]
+
+if False:
+	# When an implementation does make it,
+	# re-enable these tests.
+	type_samples.append((
+		'inet', [
+			IPAddress4('255.255.255.255'),
+			IPAddress4('127.0.0.1'),
+			IPAddress4('10.0.0.1'),
+			IPAddress4('0.0.0.0'),
+			IPAddress6('::1'),
+			IPAddress6('ffff' + ':ffff'*7),
+			IPAddress6('fe80::1'),
+			IPAddress6('fe80::1'),
+			IPAddress6('0::0'),
+		],
+	))
+	type_samples.append((
+		'cidr', [
+			IPNetwork4('255.255.255.255/32'),
+			IPNetwork4('127.0.0.0/8'),
+			IPNetwork4('127.1.0.0/16'),
+			IPNetwork4('10.0.0.0/32'),
+			IPNetwork4('0.0.0.0/0'),
+			IPNetwork6('ffff' + ':ffff'*7 + '/128'),
+			IPNetwork6('::1/128'),
+			IPNetwork6('fe80::1/128'),
+			IPNetwork6('fe80::0/64'),
+			IPNetwork6('fe80::0/16'),
+			IPNetwork6('0::0/0'),
+		],
+	))
 
 class test_driver(pg_unittest.TestCaseWithCluster):
 	"""
@@ -296,6 +329,31 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 				raise v
 		self.failUnlessRaises(pg_exc.QueryCanceledError, raise_exc, rl)
 
+	def testClones(self):
+		self.db.execute('create table _can_clone_see_this (i int);')
+		try:
+			with self.db.clone() as db2:
+				self.failUnlessEqual(db2.prepare('select 1').first(), 1)
+				self.failUnlessEqual(db2.prepare(
+						"select count(*) FROM information_schema.tables " \
+						"where table_name = '_can_clone_see_this'"
+					).first(), 1
+				)
+		finally:
+			self.db.execute('drop table _can_clone_see_this')
+		# check already open
+		db = self.db.clone()
+		self.failUnlessEqual(db.prepare('select 1').first(), 1)
+		db.close()
+
+		ps = self.db.prepare('select 1')
+		ps2 = ps.clone()
+		self.failUnlessEqual(ps2.first(), ps.first())
+		ps2.close()
+		c = ps.declare()
+		c2 = c.clone()
+		self.failUnlessEqual(c.read(), c2.read())
+
 	def testItsClosed(self):
 		ps = self.db.prepare("SELECT 1")
 		# If scroll is False it will pre-fetch, and no error will be thrown.
@@ -322,7 +380,7 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 		ps = self.db.prepare('select 1')
 		sid = ps.statement_id
 		ci = ps.chunks()
-		ci_id = ci.cursor.cursor_id
+		ci_id = ci.cursor_id
 		c = ps.declare()
 		cid = c.cursor_id
 		# make sure there are no remaining xact references..
@@ -331,13 +389,13 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 		# be removed before we can observe the effects __del__
 		del c
 		gc.collect()
-		self.failUnless(self.db.typio.encode(cid) in self.db._closeportals)
+		self.failUnless(self.db.typio.encode(cid) in self.db.pq.garbage_cursors)
 		del ci
 		gc.collect()
-		self.failUnless(self.db.typio.encode(ci_id) in self.db._closeportals)
+		self.failUnless(self.db.typio.encode(ci_id) in self.db.pq.garbage_cursors)
 		del ps
 		gc.collect()
-		self.failUnless(self.db.typio.encode(sid) in self.db._closestatements)
+		self.failUnless(self.db.typio.encode(sid) in self.db.pq.garbage_statements)
 
 	def testStatementCall(self):
 		ps = self.db.prepare("SELECT 1")
@@ -538,7 +596,7 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 		with self.db.xact():
 			self.db.execute("CREATE TABLE foo (i int)")
 			foo = self.db.prepare('insert into foo values ($1)')
-			foo.load(((x,) for x in range(500)))
+			foo.load_rows(((x,) for x in range(500)))
 
 			copy_foo = self.db.prepare('copy foo to stdout')
 			foo_content = set(copy_foo)
@@ -553,7 +611,7 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 		with self.db.xact():
 			self.db.execute("CREATE TABLE foo (i int)")
 			foo = self.db.prepare('copy foo from stdin')
-			foo.load((str(i).encode('ascii') + b'\n' for i in range(200)))
+			foo.load_rows((str(i).encode('ascii') + b'\n' for i in range(200)))
 			foo_content = list((
 				x for (x,) in self.db.prepare('select * from foo order by 1 ASC')
 			))
@@ -587,8 +645,8 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 			"CREATE OR REPLACE FUNCTION ifoo(int) RETURNS int LANGUAGE SQL AS 'select $1'"
 		)
 		ifoo = self.db.proc('ifoo(int)')
-		self.failUnless(ifoo(1) == 1)
-		self.failUnless(ifoo(None) is None)
+		self.failUnlessEqual(ifoo(1), 1)
+		self.failUnlessEqual(ifoo(None), None)
 		self.db.execute(
 			"CREATE OR REPLACE FUNCTION ifoo(varchar) RETURNS text LANGUAGE SQL AS 'select $1'"
 		)
@@ -742,10 +800,10 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 		# make sure it's not cheating
 		self.failUnlessEqual(c.cursor_id, cid)
 
-	def testChunking(self):
+	def testLoadRows(self):
 		gs = self.db.prepare("SELECT i FROM generate_series(1, 10000) AS g(i)")
 		self.failUnlessEqual(
-			list((x[0] for x in chain.from_iterable(gs.chunks()))),
+			list((x[0] for x in gs.rows())),
 			list(range(1, 10001))
 		)
 		# exercise ``for x in chunks: dst.load(x)``
@@ -757,44 +815,8 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 				FROM generate_series(1, 10000) g(i);
 				"""
 			)
-			read = self.db.prepare('select * FROM chunking').chunks(chunksize = 256)
-			write = db2.prepare('insert into chunking values ($1, $2)').load
-			with db2.xact():
-				for rows in read:
-					write(rows)
-			del read, write
-
-			self.failUnlessEqual(
-				self.db.prepare('select count(*) FROM chunking').first(),
-				20000
-			)
-			self.failUnlessEqual(
-				self.db.prepare('select count(DISTINCT i) FROM chunking').first(),
-				10000
-			)
-		self.db.execute('DROP TABLE chunking')
-
-	def testChunkingInXact(self):
-		with self.db.xact():
-			self.testChunking()
-
-	def testChunkDetect(self):
-		gs = self.db.prepare("SELECT i FROM generate_series(1, 10000) AS g(i)")
-		self.failUnlessEqual(
-			list((x[0] for x in chain.from_iterable(gs.chunks()))),
-			list(range(1, 10001))
-		)
-		# exercise ``for x in chunks: dst.load(x)``
-		with self.db.connector() as db2:
-			db2.execute(
-				"""
-				CREATE TABLE chunking AS
-				SELECT i::text AS t, i::int AS i
-				FROM generate_series(1, 10000) g(i);
-				"""
-			)
-			read = self.db.prepare('select * FROM chunking').chunks(chunksize = 256)
-			write = db2.prepare('insert into chunking values ($1, $2)').load
+			read = self.db.prepare('select * FROM chunking').rows()
+			write = db2.prepare('insert into chunking values ($1, $2)').load_rows
 			with db2.xact():
 				write(read)
 			del read, write
@@ -809,9 +831,44 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 			)
 		self.db.execute('DROP TABLE chunking')
 
-	def testChunkDetectInXact(self):
+	def testLoadRowsInXact(self):
 		with self.db.xact():
-			self.testChunking()
+			self.testLoadRows()
+
+	def testLoadChunk(self):
+		gs = self.db.prepare("SELECT i FROM generate_series(1, 10000) AS g(i)")
+		self.failUnlessEqual(
+			list((x[0] for x in chain.from_iterable(gs.chunks()))),
+			list(range(1, 10001))
+		)
+		# exercise ``for x in chunks: dst.load_chunks(x)``
+		with self.db.connector() as db2:
+			db2.execute(
+				"""
+				CREATE TABLE chunking AS
+				SELECT i::text AS t, i::int AS i
+				FROM generate_series(1, 10000) g(i);
+				"""
+			)
+			read = self.db.prepare('select * FROM chunking').chunks()
+			write = db2.prepare('insert into chunking values ($1, $2)').load_chunks
+			with db2.xact():
+				write(read)
+			del read, write
+
+			self.failUnlessEqual(
+				self.db.prepare('select count(*) FROM chunking').first(),
+				20000
+			)
+			self.failUnlessEqual(
+				self.db.prepare('select count(DISTINCT i) FROM chunking').first(),
+				10000
+			)
+		self.db.execute('DROP TABLE chunking')
+
+	def testLoadChunkInXact(self):
+		with self.db.xact():
+			self.testLoadChunk()
 
 	def testSimpleDML(self):
 		self.db.execute("CREATE TEMP TABLE emp(emp_name text, emp_age int)")
@@ -870,12 +927,9 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 			mset = (
 				(2,), (2,), (3,), (4,), (5,),
 			)
-			insert_t.load(mset)
-			self.failUnlessEqual(mset, tuple([
-				tuple(x) for x in self.db.prepare(
-					"SELECT * FROM t ORDER BY 1 ASC"
-				)
-			]))
+			insert_t.load_rows(mset)
+			content = self.db.prepare("SELECT * FROM t ORDER BY 1 ASC")
+			self.failUnlessEqual(mset, tuple(content()))
 		finally:
 			self.db.execute("DROP TABLE t")
 
@@ -890,7 +944,7 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 				"SELECT $1::" + typname
 			)
 			for sample in sample_data:
-				rsample = pb.first(sample)
+				rsample = list(pb.rows(sample))[0][0]
 				if isinstance(rsample, pg_types.Array):
 					rsample = rsample.nest()
 				self.failUnless(
@@ -961,9 +1015,9 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 				# Expecting a double TupleError here, one from the composite pack
 				# and one from the row pack.
 				self.failUnless(isinstance(err.__context__, pg_exc.ColumnError))
-				self.failUnlessEqual(err.index, 0)
+				self.failUnlessEqual(int(err.details['position']), 0)
 				# attribute number that the failure occurred on
-				self.failUnlessEqual(err.__context__.index, 0)
+				self.failUnlessEqual(int(err.__context__.details['position']), 0)
 			else:
 				self.fail("failed to raise TupleError")
 
@@ -988,7 +1042,7 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 			except pg_exc.ColumnError as err:
 				self.failUnless(isinstance(err.__context__, ThisError))
 				# might be too inquisitive....
-				self.failUnlessEqual(err.index, 0)
+				self.failUnlessEqual(int(err.details['position']), 0)
 				self.failUnless('numeric' in err.message)
 				self.failUnless('col' in err.message)
 			else:
@@ -1000,8 +1054,8 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 				self.failUnless(isinstance(err.__context__, pg_exc.ColumnError))
 				self.failUnless(isinstance(err.__context__.__context__, ThisError))
 				# might be too inquisitive....
-				self.failUnlessEqual(err.index, 0)
-				self.failUnlessEqual(err.__context__.index, 0)
+				self.failUnlessEqual(int(err.details['position']), 0)
+				self.failUnlessEqual(int(err.__context__.details['position']), 0)
 				self.failUnless('test_tuple_error' in err.message)
 			else:
 				self.fail("failed to raise TupleError from reception")
@@ -1009,52 +1063,59 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 			self.db.execute('drop type test_tuple_error;')
 
 	def testSyntaxError(self):
-		self.failUnlessRaises(
-			pg_exc.SyntaxError,
-			self.db.prepare("SELEKT 1")
-		)
+		try:
+			self.db.prepare("SELEKT 1")()
+		except pg_exc.SyntaxError:
+			return
+		self.fail("SyntaxError was not raised")
 
 	def testSchemaNameError(self):
-		self.failUnlessRaises(
-			pg_exc.SchemaNameError,
-			self.db.prepare("SELECT * FROM sdkfldasjfdskljZknvson.foo")
-		)
+		try:
+			self.db.prepare("SELECT * FROM sdkfldasjfdskljZknvson.foo")()
+		except pg_exc.SchemaNameError:
+			return
+		self.fail("SchemaNameError was not raised")
 
 	def testUndefinedTableError(self):
-		self.failUnlessRaises(
-			pg_exc.UndefinedTableError,
-			self.db.prepare("SELECT * FROM public.lkansdkvsndlvksdvnlsdkvnsdlvk")
-		)
+		try:
+			self.db.prepare("SELECT * FROM public.lkansdkvsndlvksdvnlsdkvnsdlvk")()
+		except pg_exc.UndefinedTableError:
+			return
+		self.fail("UndefinedTableError was not raised")
 
 	def testUndefinedColumnError(self):
-		self.failUnlessRaises(
-			pg_exc.UndefinedColumnError,
-			self.db.prepare("SELECT x____ysldvndsnkv FROM information_schema.tables")
-		)
+		try:
+			self.db.prepare("SELECT x____ysldvndsnkv FROM information_schema.tables")()
+		except pg_exc.UndefinedColumnError:
+			return
+		self.fail("UndefinedColumnError was not raised")
 
 	def testSEARVError_avgInWhere(self):
-		self.failUnlessRaises(
-			pg_exc.SEARVError,
-			self.db.prepare("SELECT 1 WHERE avg(1) = 1")
-		)
+		try:
+			self.db.prepare("SELECT 1 WHERE avg(1) = 1")()
+		except pg_exc.SEARVError:
+			return
+		self.fail("SEARVError was not raised")
 
 	def testSEARVError_groupByAgg(self):
-		self.failUnlessRaises(
-			pg_exc.SEARVError,
-			self.db.prepare("SELECT 1 GROUP BY avg(1)")
-		)
+		try:
+			self.db.prepare("SELECT 1 GROUP BY avg(1)")()
+		except pg_exc.SEARVError:
+			return
+		self.fail("SEARVError was not raised")
 
 	def testTypeMismatchError(self):
-		self.failUnlessRaises(
-			pg_exc.TypeMismatchError,
-			self.db.prepare("SELECT 1 WHERE 1")
-		)
+		try:
+			self.db.prepare("SELECT 1 WHERE 1")()
+		except pg_exc.TypeMismatchError:
+			return
+		self.fail("TypeMismatchError was not raised")
 
 	def testUndefinedObjectError(self):
 		try:
 			self.failUnlessRaises(
 				pg_exc.UndefinedObjectError,
-				self.db.prepare("CREATE TABLE lksvdnvsdlksnv(i intt___t)")
+				self.db.prepare, "CREATE TABLE lksvdnvsdlksnv(i intt___t)"
 			)
 		except:
 			# newer versions throw the exception on execution
@@ -1077,7 +1138,7 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 		self.db.execute("DROP TABLE withfoo")
 		self.failUnlessRaises(
 			pg_exc.UndefinedTableError,
-			self.db.prepare("SELECT * FROM withfoo")
+			self.db.execute, "SELECT * FROM withfoo"
 		)
 
 	def testTransactionAbort(self):
@@ -1091,7 +1152,7 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 			pass
 		self.failUnlessRaises(
 			pg_exc.UndefinedTableError,
-			self.db.prepare("SELECT * FROM withfoo")
+			self.db.execute, "SELECT * FROM withfoo"
 		)
 
 	def testPreparedTransactionCommit(self):
@@ -1147,8 +1208,9 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 		self.db.execute("create table gidtable as select 'foo'::text as t;")
 		x.prepare()
 		x.rollback()
-		self.failUnlessRaises(pg_exc.UndefinedTableError,
-			self.db.prepare("select * from gidtable")
+		self.failUnlessRaises(
+			pg_exc.UndefinedTableError,
+			self.db.execute, "select * from gidtable"
 		)
 
 	def testPreparedTransactionRecovery(self):
@@ -1171,8 +1233,9 @@ class test_driver(pg_unittest.TestCaseWithCluster):
 		x = self.db.xact(gid='recover dis abort')
 		x.recover()
 		x.rollback()
-		self.failUnlessRaises(pg_exc.UndefinedTableError,
-			self.db.prepare("select * from distableabort")
+		self.failUnlessRaises(
+			pg_exc.UndefinedTableError,
+			self.db.execute, "select * from distableabort"
 		)
 
 	def testPreparedTransactionFailedRecovery(self):
